"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const BitBuffer_1 = __importDefault(require("./BitBuffer"));
const Mapper_1 = __importDefault(require("./Mapper"));
const permutation_1 = require("../constants/permutation");
class Transformer {
    static Fk(key, ip) {
        // expand by permutation
        const expanded = Mapper_1.default.MapPermutation(permutation_1.Permutation.EP, ip.right);
        // xor with key
        const xor = expanded.xor(key);
        // map to sboxes
        const sboxMapped = Mapper_1.default.MapWithSboxs(xor);
        // map to permutation p4
        const p4 = Mapper_1.default.MapPermutation(permutation_1.Permutation.P4, sboxMapped);
        // xor p4 with left half of initial permutation
        const leftHalf = ip.left.xor(p4);
        const root = {
            EP: expanded.toString(),
            XOR_1: xor.toString(),
            Sbox: sboxMapped.toString(),
            P4: p4.toString(),
            XOR_2: leftHalf.toString(),
        };
        return [leftHalf, root];
    }
    static Transform(cipher, keys, decrypt) {
        // keys
        const key1 = decrypt ? keys.key2 : keys.key1;
        const key2 = decrypt ? keys.key1 : keys.key2;
        // initial permutation
        const ip = Mapper_1.default.MapPermutation(permutation_1.Permutation.IP, cipher);
        // call fk function
        const fk1 = Transformer.Fk(key1, ip);
        // swap variables
        const swap = new BitBuffer_1.default(ip.right.buffer, fk1[0].buffer);
        // call fk function
        const fk2 = Transformer.Fk(key2, swap);
        // combine ip right half and fk result
        const combined = new BitBuffer_1.default(fk2[0].buffer, swap.right.buffer);
        // ip inverse
        const ipInverse = Mapper_1.default.MapPermutation(permutation_1.Permutation.IP_1, combined);
        const root = {
            IP: ip.toString(),
            FK_1: fk1[1],
            SW: swap.toString(),
            FK_2: fk2[1],
            Combined: combined.toString(),
            IP_1: ipInverse.toString(),
        };
        return [ipInverse, root];
    }
    static Decrypt(cipher, keys) {
        return Transformer.Transform(cipher, keys, true);
    }
    static Encrypt(plain, keys) {
        return Transformer.Transform(plain, keys, false);
    }
}
exports.default = Transformer;
